#!/usr/bin/perl

use XML::Parser;

# This script is used to verify that networking is configured properly. It verifies the network configuration by the following:
#
# 0) Discover the network configuration from OVF environment
# 1) Discover the IP address of eth0 from ifconfig
# 2) Discover the gateway address from route
# 3) Ping the gateway
#
# If any of these steps fail, the network is considered not working and the user is prompted to configure static networking.
# After the prompted values are confirmed, the configuration is applied and the network configuration is tested again.

# Pattern with capturing group for matching a valid IP address
$IP_PATTERN = "((?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))";

# Pattern with capturing group for matching a valid fully qualified domain name hostname
$FQDN_PATTERN = "((?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9\\-]*[a-zA-Z0-9])?)\\.)+(?:[A-Za-z](?:[a-zA-Z0-9\\-]*[a-zA-Z0-9])?))";

# Pattern with IP's capturing group for finding the gateway address from the "route -n" command output
$ROUTE_GATEWAY_PATTERN = "0\.0\.0\.0\\s*$IP_PATTERN\\s*.*UG.*eth0";

# Pattern for determining whether the ping was successful from the ping command output
$PING_PATTERN = "1 received";

#Asks the user a yes/no question
#Returns 1 for yes, 0 for no
sub user_confirm {
    my $question = shift @_;
    my $default = shift @_;

    while(1) {
        print "$question [$default]: ";

        my $answer = <STDIN>;
        chomp $answer;

        #if they didnt enter anything, use the default
        if($answer eq "") {
            $answer = $default;
        }

        #figure out if they entered yes or no
        if ($answer =~ /^(y|yes)$/i) {
            return 1;
        } elsif ($answer =~ /^(n|no)$/i) {
            return 0;
        } else {
            print "Please answer yes or no.\n";
        }
    }
}

#
# This method will check the link status of eth0.
#
#     "Link detected: no"
#     "Link detected: yes"
#
# return 1 means link detected 0 means no link detected
#
sub checkLinkStatus {
   my $result = 0; # Default to failed

   # Execute the ethtool command to search for the link status
   open(ETHTOOL, "/sbin/ethtool eth0 |") or die "Failed to execute ethtool";
   while (<ETHTOOL>) {
      # Look for the link status
      if ($_ =~ m/Link detected: yes/) {
          $result = 1;
          break;
      }
   }
   close(ETHTOOL);

#if ( $result ) {
#print "Link has been detected\n";
#} else {
#print "Link was not detected\n";
#}

   return $result;
}

#
# Check that networking is up and running successfully by detecting the system's
# IP address, and pinging the gateway.
#
# @return 0 if the network is not working, 1 if it is working
#
sub checkNetwork {
   my $result = 0; # Default to failed

   print "\nChecking network configuration. Please wait...\n";

   # Execute the ifconfig eth0 command to search for the IP address
   open(IFCONFIG, "ifconfig eth0 |") or die "Failed to execute ifconfig";
   my $ipAddress = "";
   while (<IFCONFIG>) {
      # Look for the IPV4 address
      if ($_ =~ /inet addr:$IP_PATTERN/) {
          $ipAddress = $1;
          break;
      }
   }
   close(IFCONFIG);

   # If the address was not found, the network is invalid
   if ($ipAddress eq "") {
       return $result;
   }

   # Get the gateway from the route and then ping it
   open(ROUTE, "/sbin/route -n|") or die "Failed to execute route";
   my $gateway = "";
   while (<ROUTE>) {
       # Look for the Gateway address
       if ($_ =~ /$ROUTE_GATEWAY_PATTERN/) {
	   $gateway = $1;
	   break;
       }
   }
   close(ROUTE);

   # check if we got a gateway address
   if ($gateway eq "") {
       print "\nNo default route has been configured.\n";
       return $result;
   }

   # Ping the gateway address to see if it responds
   open(PING, "ping -c 1 -w 1 $gateway |") or die "Failed to execute ping";
   my $success = 0;
   while (<PING>) {
       # Look for reponse
       if ($_ =~ /$PING_PATTERN/) {
	   $success = 1;
	   break;
       }
   }
   close(PING);

   # check if we got a response
   if ($success == 0) {
       print "\nUnable to reach gateway at '$gateway'.\n";
       return $result;
   }
 
   return 1;
}

#
# If we got here, then that means that the network is not setup correctly.
# @return 1 if the network is static , 0 if dhcp
#
sub checkNetworkConfig {
   my $result = 1; # Default to static config

   # Look for the eth0 in the interfaces for and see how it is configured
   open(IFTYPE, "cat /etc/network/interfaces | grep eth0 | grep iface |") or die "Failed to find network file";
   while (<IFTYPE>) {
      # What type is the interface set for dhcp or static?
      if ($_ =~ m/dhcp/) {
          $result = 0;
      }
   }
   close(IFTYPE);

#if ( $result == 0 ) {
#       print "IF type is dhcp\n";
#   } else {
#       print "IF type is static\n";
#   }

   return $result;
}

#
# Function to remove whitespaces before and after a string
#
sub trim($) {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

#
# Prompt the user to enter an IPv4 address.
#
# @param promptText - the text to use as the prompt for the IP address
# @param required - to see if we require an entry for this.
# @return the valid IP address
#
sub promptForIP {
   my ($promptText, $isRequired) = @_;

   my $value = "";
   while (!$value) {
      print "$promptText: ";
      my $input = <STDIN>;
      chomp($input);
      $input = trim($input);
      if ( $input eq "" && $isRequired eq "false" ) {
          return $input;
      } else {
          if (validateIP($input)) {
             $value = $input;
          } else {
             print "Invalid IP address. Please enter a valid value. For example: 192.0.0.1\n";
          }
      }
   }
   return $value;
}

#
# Validate that the IP address matches the expected IPv4 format. 
#
# @param ip - the ip to validate
# @return 1 if the ip is valid, 0 if not
#
sub validateIP {
   my ($ip) = @_;

   if ($ip =~ /$IP_PATTERN/) {
      if ($ip eq $1) {
	return 1;
      }
   }
   return 0;
}


#
# Prompt the user to enter the static networking configuration
#
# @return %config - map containing the network configuration settings
#
sub promptForNetworkConfig {
   my %config;

   print "\n";
   print "Static Configuration\n";
   print "--------------------\n";
   $config{hostName} = promptForHostName("Host name            ");
   $config{ipAddress} =      promptForIP("Host IP address      ", "true");
   $config{netMask} =        promptForIP("Net mask             ", "true");
   $config{gateway} =        promptForIP("Gateway              ", "true");
   $config{primaryDNS} =     promptForIP("Primary DNS address  ", "false");
   $config{secondaryDNS} =   promptForIP("Secondary DNS address", "false");

   %config = calculateDerivedConfigValues(%config);

   $config{ipAllocPolicy} = 'STATIC';
   $config{searchDomains} = promptForAdditionalDomains(%config);

   return %config;
}

#
# Prompt the user to enter the host name of the system. 
#
# @param promptText - the text to use as the prompt for the host name
# @return the valid fully qualified host name
#
sub promptForHostName {
   my ($promptText) = @_;

   while (1) {
      print "$promptText: ";
      my $input = <STDIN>;
      chomp($input);
      if(validateHostname($input)) {
          return $input;
      } else {
          print "That does not appear to be a valid hostname\n";
      }
   }
}

#returns true if input arg is a valid hostname
sub validateHostname {
    my $hostname = $_[0];

    if ($hostname =~ /[^-A-Za-z0-9]/ || length($hostname) > 63
        || length($hostname) <= 0) {
	return 0;
    }
    return 1;
}

#
# Validate that the FQDN host name matches the expected format. The host name 
# should be in the form of '<hostname>.<domain>' with as many dotted segments 
# in the domain portion as necessary.
#
# @param hostName - the name to validate
# @return 1 if the hostName is valid, 0 if not
#
sub validateFqdnHostName {
   my ($hostName) = @_;
   
   if ($hostName =~ /$FQDN_PATTERN/) {
       if ($hostName eq $1) {
          return 1;
       }
   }
   return 0;
}

#
# Validate if the specified list of search domains are valid
#
# @param domains - space separated list of search domains
# @return 1 = valid, 0 = invalid, @searchDomains.
#
sub validateSearchDomains {
    my ($domainlist) = @_;

    my $valid = 1;
    my $searchDomains = "";

    if ($domainlist ne "") {
	# validate the domains
	@domains = split(' ', $domainlist);
	foreach $domain (@domains) {
            if (!validateFqdnHostName($domain)) {
                print "Invalid search domain: $domain\n";
                $valid = 0;
		last;
            } else {
		# If domain was valid, append it to the search domains value
		$searchDomains .= ' ' . $domain;
            }
	}
    }

    # use substr to strip space at front of string
    return($valid, substr($searchDomains, 1));
}


#
# Prompt the user to optionally specify additional search domains.
#
# @param %config - the configuration values map
# @return the search domain string, separated by spaces between the domains
#
sub promptForAdditionalDomains {
   my (%config) = @_;

   my $searchDomains = "";

   my $valid = 0;

   while (!$valid) {
      print "Please enter in all search domains separated by spaces (can be left blank):\n";
      my $input = <STDIN>;
      chomp($input);

      # if additional domains were specified
      ($valid, $searchDomains) = validateSearchDomains($input);
   }

   return $searchDomains;
}

#
# Calculate the derived configuration values from the config values the user entered at the prompts.
# The derived values include the network, broadcast, hostName and domain values in the config map.
#
# @param %config - the configuration values map
# @return %config - the configuration values map with the derived values filled in
#
sub calculateDerivedConfigValues {
   my (%config) = @_;

   # Calculate the network and broadcast values
   
   # Split the IP address and net mask addresses by . into 4 string segments
   my @ipSegments = split /\./, $config{ipAddress};
   my @netMaskSegments = split /\./, $config{netMask};

   my $network;
   my $broadcast;
   for ($i = 0; $i < 4; $i++) {
      # Add zero to the strings to force conversion to a number
      $ipSegment = $ipSegments[$i] + 0;
      $netMaskSegment = $netMaskSegments[$i] + 0;

      # Bitwise AND the segments together to produce the network segment
      $netSegment = $ipSegment & $netMaskSegment;

      # Bitwise OR the address segment with the inverse of the mask segment to produce the broadcast segment
      $broadcastSegment = $ipSegment | ($netMaskSegment ^ 0xFF);

      if ($i == 0) {
         $network = $netSegment;
         $broadcast = $broadcastSegment;
      } else {
         $network = $network . '.' . $netSegment;
         $broadcast = $broadcast . '.' . $broadcastSegment;
      }
   }

   # Place the calculated values into the config map
   $config{network} = $network;
   $config{broadcast} = $broadcast;

   return %config;
}

#
# Display the network configuration and prompt the user to confirm the
# settings are correct.
#
# @param %config - map of the network configuration
#
sub displayConfig {
   my (%config) = @_;

   print "\n\n";
   print "Selected IP policy    : $config{ipAllocPolicy}\n";
   print "Selected host name    : $config{hostName}\n";
   print "Selected IP           : $config{ipAddress}\n";
   print "Selected net mask     : $config{netMask}\n";
   print "Selected gateway      : $config{gateway}\n";
   print "Selected primary DNS  : $config{primaryDNS}\n";
   print "Selected secondary DNS: $config{secondaryDNS}\n";
   print "Search domains        : $config{searchDomains}\n";
   print "Calculated network    : $config{network}\n";
   print "Calculated broadcast  : $config{broadcast}\n";
}

#
# Display the network configuration and prompt the user to confirm the
# settings are correct.
#
# @param %config - map of the network configuration
# @return 1 if the user confirmed the configuration, 0 if the user 
#     rejected the configuration
#
sub confirmConfig {
   my (%config) = @_;

   displayConfig(%config);
   return user_confirm("Are these values correct?", 'y');
}

#
# Modify the system to reflect the static network configuration and 
# restart networking using the new configuration.
#
# @param %config - map of the new static network configuration
#
sub applyNetworkConfig {
   my (%config) = @_;

   my $timestamp = time();

   # Update the hostname file
   system("cp", "/etc/hostname", "/etc/hostname.$timestamp.bak");
   open(HOSTNAME, ">/etc/hostname") or die "Failed to open /etc/hostname for writing";
   print HOSTNAME "$config{hostName}\n";
   close(HOSTNAME);

   # Update the hosts file
   open(HOSTS, "/etc/hosts") or die "Failed to read from /etc/hosts";
   @hostLines = <HOSTS>;
   close(HOSTS);

   system("cp", "/etc/hosts", "/etc/hosts.$timestamp.bak");
   open(HOSTS, ">/etc/hosts") or die "Failed to open /etc/hosts for writing";
   # Add localhost info
   print HOSTS "127.0.0.1\tlocalhost\n";
   print HOSTS "127.0.0.1\t$config{hostName}\n";

   # Add any lines from the original hosts file that don't refer to localhost (127.0.0.1)
   foreach $hostLine (@hostLines) {
      if (index($hostLine, "127.0.0.1") < 0) {
         print HOSTS $hostLine;
      }
   }
   close(HOSTS);

   # Update the host name on the live system
   system("/bin/hostname", $config{hostName});

   # Stop networking (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/networking stop |");
   @cmdlines = <CMD>;
   close(CMD);

   # Read the existing interfaces file
   open(INTERFACES, "/etc/network/interfaces") or die "Failed to read from /etc/network/interfaces";
   @interfacesLines = <INTERFACES>;
   close(INTERFACES);

   system("cp", "/etc/network/interfaces", "/etc/network/interfaces.$timestamp.bak");
   open(INTERFACES, ">/etc/network/interfaces") or die "Failed to open /etc/network/interfaces for writing";
   # Rewrite the interfaces file using the existing lines, substituting the eth0 configuration line(s) with 
   # the new static configuration.
   my $inEth0 = 0;
   foreach $interfacesLine (@interfacesLines) {
      # If this line is the start of the eth0 block
      if ($interfacesLine =~ /^iface eth0/) {
         # Toggle the block marker on and skip printing the line
         $inEth0 = 1;
         next; # Continue the foreach loop
      }
      # Look for whitespace at the beginning of the line to indicate whether we're still in the
      # eth0 block
      if ($inEth0 && $interfacesLine !~ /^\s/) {
         # Detected the end of the eth0 block. Insert the new configuration lines, then toggle the block
         # marker off
         print INTERFACES "iface eth0 inet static\n";
         print INTERFACES "\taddress $config{ipAddress}\n";
         print INTERFACES "\tnetmask $config{netMask}\n";
         if ( $config{network} ne "" ) {
             print INTERFACES "\tnetwork $config{network}\n";
         }
         if ( $config{broadcast} ne "" ) {
             print INTERFACES "\tbroadcast $config{broadcast}\n";
         }
         if ( $config{gateway} ne "" ) {
             print INTERFACES "\tgateway $config{gateway}\n";
         }
         if ( $config{domain} ne "" ) {
             print INTERFACES "\tdns-domain $config{domain}\n";
         }
         if ( $config{searchDomains} ne "" ) {
             # This allows compatibility with AMT (this change was easier)
             open(TAIL, ">/etc/resolvconf/resolv.conf.d/tail") or die "Failed to open /etc/resolvconf/resolv.conf.d/tail for writing";
             print TAIL "search $config{searchDomains}\n";
             close(TAIL);
         }

         my $dns_servers = "";

         if ( $config{primaryDNS} ne  "") {
              $dns_servers .= $config{primaryDNS} . " ";
         }

         if( $config{secondaryDNS} != "" ) {
              $dns_servers .= $config{secondaryDNS} . " ";
         }

         if($dns_servers ne "") {
               print INTERFACES "\tdns_nameservers $dns_servers\n";
	 }

         $inEth0 = 0;
      }
      # If we're not in the eth0 block, print the line from the original interfaces file
      if (! $inEth0) {
         print INTERFACES $interfacesLine;
      }
   }
   close(INTERFACES);

   # Start networking with the new configuration (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/networking start |");
   @cmdlines = <CMD>;
   close(CMD);

   # Restart VMware Tools (use open/close rather than system to swallow command output)
   open(CMD, "/etc/init.d/vmware-tools restart |");
   @cmdlines = <CMD>;
   close(CMD);

   # Send a ping to advertise ourselves
   open(CMD, "/bin/ping -c 1 " . $config{gateway} ." |");
   @cmdlines = <CMD>;
   close(CMD);
}

#
# Setup network configuration from OVF values
# @param - none
# @return - none
#
sub setupStaticIp {

    # While the network configuration is invalid
    $forceFail = 0;
    while (!$valid) {
        my %config;
        my $confirmed = 0;

        # Prompt for the network configuration and loop until it's confirmed
        while (!$confirmed) {
            %config = promptForNetworkConfig();
            $confirmed = confirmConfig(%config);
        }

        # Apply the network configuration, and loop back to testing the network configuration
        print "\n\nApplying network configuration. Please wait...\n";
        applyNetworkConfig(%config);

        # Check the network validity with the new configuration
        $valid = checkNetwork();

        # If the configuration is still invalid, prompt the user if they want to force use these settings.
        # This will allow the user to exit the loop if for some reason the settings are correct, but
        # we still can't ping the gateway.
        if (!$valid) {
            if (user_confirm("\nThe network configuration is still invalid.\nAre you sure you want to use these settings?", 'n')) {
                $valid = 1;
                $forceFail = 1;
            }
        }
    }

    if ( $forceFail ) {
        print "\n";
	print "You have chosen to enable Flash Accel Management Console with a possible network\n";
	print "configuration error. If you are unable to access the web interface, or it does not\n";
	print "function correctly, please resolve outstanding networking issues and restart this\n";
	print "appliance.\n";
	print "\n";
        exit(1);
    }
}

my %netconfig;

#
# OVF environment XML processing handlers
#
sub OVF_start_handler {
    my $p  = shift;
    my $el = shift;
    while (my $key = shift) {
	my $val = shift;
	if ($key == "oe.key" && $val =~ /flashaccel/) {
	    my $attr = $val;
	    $key = shift;
	    $val = shift;
	    if ($key == "oe.value") {
		$attr =~ s/flashaccel\.//;
		$netconfig{$attr} = $val;
	    }
	}
    }
}

sub OVF_end_handler { }
sub Default_handler { }

#
# Setup network configuration from OVF values
# @param - none
# @return - "dhcp" or "static" or "error"
#
sub setupStaticFromOVF {

    # Read the ovf environment
    open(OVF, "/usr/sbin/vmtoolsd --cmd 'info-get guestinfo.ovfenv' |") or die "Failed to execute vmtools command";
    @ovfenv = <OVF>;
    close(OVF);

    # parse the OVF xml content
    my $parser = new XML::Parser();
    $parser->setHandlers (Start => \&OVF_start_handler,
			  End => \&OVF_end_handler,
			  Default => \&Default_handler);
    $parser->parse(join("\n", @ovfenv));

    # Check that all 6 STATIC fields are blank for DHCP
    if ( $netconfig{hostName} eq '' and $netconfig{ipAddress} eq '' and $netconfig{netMask} eq '' and $netconfig{gateway} eq '' and $netconfig{primaryDNS} eq '' and $netconfig{secondaryDNS} eq '' ) {

        # We have DHCP return back immediately
        $netconfig{ipAllocPolicy} = 'DHCP';
	return 'dhcp';

    # Check that the 5 required STATIC fields are not blank
    } elsif ( $netconfig{hostName} and $netconfig{ipAddress} and $netconfig{netMask} and $netconfig{gateway} and $netconfig{primaryDNS} ) {

        # We have the 5 fields to attempt a STATIC configuration
        $netconfig{ipAllocPolicy} = 'STATIC';

    } else {

        # Too many fileds for DHCP, not enough for STATIC
	return "error";
    }

    # Check if the received values are ok
    if (!validateHostname($netconfig{hostName})) {
	print "\nHost name '" . $netconfig{hostName} . "' is invalid.\n";
	return "error";
    }

    # Check all the IP values passed in
    if (!validateIP($netconfig{ipAddress})) {
	print "IP address '" . $netconfig{ipAddress} . "' is invalid.\n";
	return "error";
    }
    if (!validateIP($netconfig{netMask})) {
	print "Netmask address '" . $netconfig{netMask} . "' is invalid.\n";
	return "error";
    }
    if (!validateIP($netconfig{gateway})) {
	print "Gateway address '" . $netconfig{gateway} . "' is invalid.\n";
	return "error";
    }
    if (!validateIP($netconfig{primaryDNS})) {
	print "Primary DNS address '" . $netconfig{primaryDNS} . "' is invalid.\n";
	return "error";
    }
    if (!validateIP($netconfig{secondaryDNS})) {
	print "Secondary DNS address '" . $netconfig{secondaryDNS} . "' is invalid.\n";
	$netconfig{secondaryDNS} = "";
    }

    # Check the search domain list
    my ($dom_valid, $searchDomains) = validateSearchDomains($netconfig{searchDomains});
    if (!dom_valid) {
	return "error";
    }

    # Compute the derived values
    %netconfig = calculateDerivedConfigValues(%netconfig);

    # Apply the network configuration, and loop back to testing the network configuration
    print "\n\nApplying network configuration. Please wait...\n";
    applyNetworkConfig(%netconfig);

    # Check the network validity with the new configuration
    $valid = checkNetwork();

    # If the configuration is invalid, report it to the user and force them to re-enter the values 
    # on the console
    if (!$valid) {
	if (user_confirm("The network configuration has errors. Specified attributes may be invalid.\nAre you sure you want to use these settings?", 'n')) {
	    $valid = 1;
	    $forceFail = 1;
	} else {
	    return "error";
	}
    }

    if ( $forceFail ) {
        print "\n";
	print "You have chosen to enable Flash Accel Management Console with a possible network\n";
	print "configuration error. If you are unable to access the web interface, or it does not\n";
	print "function correctly, please resolve outstanding networking issues and restart this\n";
	print "appliance.\n";
        print "\n";
        exit(1);
    }

    # Display the values that were passed in
    print "Successfully configured the manangement interface using the following attributes. \n";
    displayConfig(%netconfig);

    return "static";
}

##################################################################################################################
# Main execution
##################################################################################################################
# At boot, the appliance's startup messages are supressed by redirecting them to an alternate console /dev/ttyS0
# For the interactive portions of this program, we'll need to explicitly switch STDIN and STDOUT to use the real console at /dev/tty1
open(STDIN, "</dev/tty1");
open(STDOUT, ">>/dev/tty1");

# Clear the screen
print (((" " x 80) . "\n") x 25);

# Check to see if the interface actually has link detected.  If not then the user must not have
# setup their network card correctly on the virtual network adapter.
$hasLink = checkLinkStatus();
if ($hasLink == 0) {
    print "\n";
    print "The virtual adapter is reporting that it cannot detect a link. This is either \n";
    print "due to a virtual machine configuration issue or network issue.  Please check\n";
    print "the network adapter settings to make sure that the \"Device Status\" is set to:\n";
    print "          o - \"Connected\"\n";
    print "          o - \"Connect at power on\"\n";
    print "\n";
    print "Flash Accel Management Console will not function until this is addressed.\b";
    print "Once resolved, please restart this appliance.\n";
    print "\n";
    print "                    <Please press enter to continue>";
    $response = <STDIN>;
    print "\n";
    exit(1);
}

#This env var is set if the network is "unconfigured"
#ask them which method they would like to use
if (-e "/var/lib/fmc/.run_network_config") {
    unlink("/var/lib/fmc/.run_network_config");

    # Read the OVF environment
    $config = setupStaticFromOVF();

    if ($config eq "static") {
	# This will only print if there's a valid static configuration
	print "\nUsing a STATIC network interface configuration.\n";
	print "To make networking changes, login to the Advanced Management Terminal (AMT).\n";
        exit(0);
    } elsif ($config eq "dhcp") {
	print "\nUsing DHCP for network interface configuration\n";
    } elsif ($config eq "error") {
	print "\n";
	print "The network configuration specified during deployment has errors. This could be\n";
	print "due to either an issue with network configuration or virtual network adapter\n";
	print "configuration.\n";
	print "\n";
	if (!user_confirm("Would you like to correct the static configuration?", 'y')) {
	    print "\n";
	    print "You have chosen to enable Flash Accel Management Console with a possible network\n";
	    print "configuration error. If you are unable to access the web interface or it does not\n";
	    print "function correctly, pleased resolve outstanding networking issues with the\n";
	    print "Advanced Management Terminal (AMT).\n";
	    print "\n";
	    print "Press any key to continue.";
	    $continue = <STDIN>;
	    exit(1);
	}
	setupStaticIp();
	exit(0);
    }
}

# Getting here means that connection is ok. Now do the initial check of the network validity and exit immediately 
# if the the network is valid.
$valid = checkNetwork();
if ($valid) {
    # This will only print if there's a valid dhcp configuration
    print "To make networking changes, login to the Advanced Management Terminal (AMT).\n";
    exit(0);
}


# The only way to truly get here is because we are in DHCP mode and we did not get back
# a response from the server.  So let the user know the situation and then ask if they 
# want to set to static mode.  Verify first
#
$isStaticConfig = checkNetworkConfig();
if ($isStaticConfig) {
    exit(0);
}

# We are in DHCP mode, notify the user and then ask if they want to set to static mode.
print "\n";
print "The manangement interface is currently configured for DHCP but the network\n";
print "could not be configured properly.  This could be due to either a DHCP \n";
print "configuration issue or virtual machine network adapter configuration.\n";
print "\n";
if (!user_confirm("Would you like to setup a static configuration?", 'y')) {
    print "\n";
    print "You have chosen to enable Flash Accel Management Console with a possible network\n";
    print "configuration error. If you are unable to access the web interface or it does not\n";
    print "function correctly, pleased resolve outstanding networking issues with the\n";
    print "Advanced Management Terminal (AMT).\n";
    print "\n";
    print "Press any key to continue.";
    $continue = <STDIN>;

    exit(1);
}

setupStaticIp();

print "\n\nContinuing system startup. Please wait...\n";

exit(0);

